%
% [Eout, Ein, roundLoss, cavVal] = ...
%    HankelFP( cavity, N, a, delITM, dPhiITM, delETM, lockMethod, phiArmIn )
%
%
% input
%   cavity : setting of the stable coupled cavity. See defCavity how settings are defined.
%   N : number of tradial points
%   a : window on ITM, RM3
%   delITM, delETM : surface deformation of ITM and ETM ( same as phiITM, size of 1 or N )
%   dPhiITM : non uniformity of ITM substrate. exp( i dPhiITM ) is
%             mutiplied when the field goes through ITM
%   lockMethod : 
%       0 : no lock
%       1 : nullify round trip phase
%       2 : nullify error signal
%   phiArm : one trip phase used as exp( - i k L ) in the arm
%            For an idealistic case, it will be -gouyPhase00 to cancel the
%            gouy phase.
%            When the lock is not 0, phiArm is offset around the locked length
%
% output
%   Eout.{fromITM, toETM, fromETM, toITM, totalRefl}
%   fromITM, toETM, fromETM and toITM are fields of type FieldVal on ITM and ETM.
%   "to" means it is arriving from the other mirror, and "from" means
%   it is leaving toward the other mirror. E.g., fromITM is the field
%   on ITM, leaving ITM toward ETM.
%   Field.E    : amplitudes at rList points
%        .Ptot : total power
%        .P00  : 00 mode power (only for fromITM field)
%   Ein : injected field, after going through ITM
%   cavVal : various field quantities in the cavity.
%            See function fillOutputs
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%{
% example
% copy and paste between %{ to %}

   Nhk = 128;  % Nfft/2
   Ahk = 0.3;  % Wfft/2 
   ROC1 = 1934; ROC2 = 2245; 
   mirrorR = 0.168; Lcav = 3994.5;
 
% test mass power transmittance
    cavity.ITM.T = 0.014;
    cavity.ETM.T = 5e-6;
% 1/ROC
    cavity.ITM.invROC = 1/ROC1;
    cavity.ETM.invROC = 1/ROC2;
% mirror radius
    cavity.ITM.radius = mirrorR;
    cavity.ETM.radius = mirrorR;
% arm loss. Make it 70ppm when no abberation is explicitly added.
    cavity.armIE.Loss = 70e-6 * 0;
% cavity length
    cavity.armIE.L = Lcav;

% scan over length
   phiArm = 0.01*(-25:25)*2*pi;

% ETM surface aberration
   lamS = 0.01;
   rList = HankelTK(Ahk, Nhk).rList;
   delETM = 1e-9 * cos( 2*pi*rList / lamS );

% calculate fields
   [Eout, Ein, roundLoss, cavVal] = ...
    HankelFP( cavity, Nhk, Ahk, 0, 0, delETM, 2, phiArm );
 
% plot power as a function of cavity length
   subplot(2,1,1)
   semilogy( phiArm, [ Eout.fromITM(:).Ptot], phiArm, [ Eout.fromITM(:).P00 ] )
% polt power as a function of raius
   subplot(2,1,2)
   ind = find( [Eout.fromITM(:).Ptot] == max([Eout.fromITM(:).Ptot]) );
   % use ind(1) in case multiple have same peak power
   semilogy( rList.^2, abs(Eout.fromITM(ind(1)).E).^2 )

%}
%
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% November 21, 2012 : Hiro Yamamoto 
%   temporal update to make it work for a simple FP
%
% May 15, 2010 Hiro Yamamoto  hiro@ligo.caltech.edu
%   first release
%   Uses HankelTK and CavityTK
%

function ...
    [Eout, Ein, roundLoss, cavVal] = ...
    HankelFP( cavity, N, a, delITM, dPhiITM, delETM, lockMethod, phiArmIn )

% calculate beam size, gouy phase and q parameter of the FP cavity
[wITM, wETM, etaArm, qArm] = calcMMParam;

% set values when not all values are specified
setDefaultInputs(nargin);
phiArmGiven; % to make this visible in this workspace

% initialize/reserve variables
initVariables;

% cavity parameters and constant fields
initCavityContents;
Cround1; % to make this visible in this workspace

% lock FP
[lockedPhi, locked] = handleLocking;
if ~locked
    warning( 'locking failed');
end

% calculate fields for each phiArm(:) values
for iphi = 1 : Nphi
    phaseFactor = exp(1i*phiArm(iphi));
    
    % fromITM = ( 1 - roundTrip*exp(2i * phase) )^-1 t1 Ein
    Eout.fromITM(iphi).E = (oneN - phaseFactor^2*Cround) \ Ein;
    
    % 3 more fiels on ITM and ETM
    Eout.toETM(iphi).E = phaseFactor * FProp * Eout.fromITM(iphi).E;
    Eout.fromETM(iphi).E = MRefETM * Eout.toETM(iphi).E;
    Eout.toITM(iphi).E = phaseFactor * FProp * Eout.fromETM(iphi).E;
    Eout.totalRefl(iphi).E = Eprompt + Mleak * Eout.toITM(iphi).E;

    % calculate powers. 00 mode of Eout.fromITM is known so calculate 00 power as well
    [ Eout.fromITM(iphi).Ptot, Eout.fromITM(iphi).P00 ] = ...
        CavityTK.calcPower( Eout.fromITM(iphi).E, E00, hkObj );
    Eout.toETM(iphi).Ptot = hkObj.hkDotProd( Eout.toETM(iphi).E, Eout.toETM(iphi).E );
    Eout.fromETM(iphi).Ptot = hkObj.hkDotProd( Eout.fromETM(iphi).E, Eout.fromETM(iphi).E );
    Eout.toITM(iphi).Ptot = hkObj.hkDotProd( Eout.toITM(iphi).E, Eout.toITM(iphi).E );
    Eout.totalRefl(iphi).Ptot = hkObj.hkDotProd( Eout.totalRefl(iphi).E, Eout.totalRefl(iphi).E );

    % rund trip loss
    Eround = Cround * Eout.fromITM(iphi).E;
    pwrAfter = hkObj.hkDotProd( Eround, Eround ) / (rITM*rETM)^2;
    roundLoss(iphi) = ( Eout.fromITM(iphi).Ptot - pwrAfter ) / ...
        Eout.fromITM(iphi).Ptot;
end

% fill cavVal if necessary
fillOutputs(nargout);

% end of main body
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% set default input params
function setDefaultInputs(nargin_)
        % round trip phase due to longitudinal part
    if nargin_ < 8
        phiArm = -etaArm;
        phiArmGiven = 0;
    else
        phiArm = phiArmIn;
        phiArmGiven = 1;
    end
    % default lock is none
    if nargin_ < 7
        lockMethod = 0;
    end
    % surface and transmission map
    if nargin_ < 6
        delETM = 0;
    end
    if nargin_ < 5
        dPhiITM = 0;
    end
    if nargin_ < 4
        delITM = 0;
    end
end

% initialize/reserve variables
function initVariables
    hkObj = HankelTK( a, N );
    oneN = diag(ones(N,1),0);

    Nphi = length( phiArm );
    roundLoss = zeros( Nphi, 1 );

    Eout.fromITM = FieldVal( N, Nphi );
    Eout.toETM = FieldVal( N, Nphi );
    Eout.fromETM = FieldVal( N, Nphi );
    Eout.toITM = FieldVal( N, Nphi );
    Eout.totalRefl = FieldVal( N, Nphi );
end

% cavity parameters
function initCavityContents
    % cavity parameters
    rITM = sqrt( 1 - cavity.ITM.T - cavity.armIE.Loss/2 );
    rETM = sqrt( 1 - cavity.ETM.T - cavity.armIE.Loss/2 );

    % reflection matrix
    MRefITM = CavityTK.MirrorFrontRef( rITM, cavity.ITM.invROC, delITM, cavity.ITM.radius, hkObj );
    MRefETM = CavityTK.MirrorFrontRef( rETM, cavity.ETM.invROC, delETM, cavity.ETM.radius, hkObj );

    Mprompt = CavityTK.MirrorBackRef( rITM, cavity.ITM.invROC, dPhiITM, cavity.ITM.radius, hkObj );
    
    Mleak = CavityTK.MirrorTrans( sqrt( cavity.ITM.T ), cavity.ITM.invROC, ...
        dPhiITM, cavity.ITM.radius, hkObj );
    
    % one path propagator
    FProp = CavityTK.PropX( cavity.armIE.L, hkObj );

    % round trip propagator
    Cround1 = FProp * MRefETM * FProp; % roundtrip without ITM reflection
    Cround = MRefITM * Cround1;

    % constant fields
    % cavity 00 mode
    E00 = CavityTK.HG00q( qArm, hkObj );

    % Elaser becomes E00 after going through ITM lens
    Elaser = CavityTK.MirrorTrans( 1, cavity.ITM.invROC, 0, 0, hkObj ) \ E00;

    % prompt reflection
    Eprompt = Mprompt * Elaser;

    % taking into account the finite aperture and transmission
    Ein = CavityTK.MirrorTrans( sqrt(cavity.ITM.T), cavity.ITM.invROC, dPhiITM, cavity.ITM.radius, hkObj ) * Elaser;
end

% front end of locking procedure
function [lockedPhi_, locked_] = handleLocking
    if (lockMethod == 1) || (lockMethod == 2)
        evalRoundTripPhase = @( phase, E0 ) angle( hkObj.hkDotProd( E0, phase^2*Cround*E0 ) );

        [lockedPhi_, locked_] = FPlock( -etaArm, evalRoundTripPhase );
    end

    if lockMethod == 2
        err2phi = -( cavity.ITM.T + cavity.ETM.T )^2 / ( 8 * cavity.ITM.T );
        evalErrorSignal = @( phase, E0 ) ...
            err2phi * imag( hkObj.hkDotProd( Eprompt, phase^2 * Mleak * Cround1 * E0 ) );

        [lockedPhi_, locked_] = FPlock( lockedPhi_, evalErrorSignal );
    end

    if lockMethod > 0
        if phiArmGiven
            phiArm = phiArm + lockedPhi_;
        else
            phiArm = lockedPhi_;
        end
    end
end

% lock FP using offset evaluator, eval( roundTripPhase, E from ITM )
function [lockedPhi, locked] = FPlock( initPhi, eval )
    cavPhi = zeros(2,2);
    phiRes = (cavity.ITM.T + cavity.ETM.T)/2 * 1e-3; % amplitude drops by 10^-3
    cavPhi(1,1) = initPhi; 
    % find locked point 10 times
    for iLoop = 1 : 10
        phi_ = exp(1i*cavPhi(1,1));
        Ecav0 = (oneN - phi_^2*Cround) \ Ein;
        cavPhi(1,2) = eval( phi_, Ecav0 );
        % if the round trip phase is small, lock done
        if ( abs(cavPhi(1,2)) <= phiRes )
            break;
        end
        del2 = cavPhi(2,2) - cavPhi(1,2);
        if (iLoop == 1) || ( abs(del2) < 1e-6 )
            cavPhiNew = cavPhi(1,1) - cavPhi(1,2);
        else
            cavPhiNew = (cavPhi(2,2)*cavPhi(1,1)-cavPhi(1,2)*cavPhi(2,1)) / del2;
        end
        cavPhi(2,1:2) = cavPhi(1,1:2);
        cavPhi(1,1) = cavPhiNew;
    end
    locked = abs(cavPhi(1,2)) <= phiRes;
    lockedPhi = cavPhi(1,1);
end

% calculate beam size on RM3, RM2 and RM1
function [wITM, wETM, etaArm, qArm] = calcMMParam
% arm cavity mode

    etaArm = MMTK.eta00FP( cavity.ITM.invROC, cavity.ETM.invROC, cavity.armIE.L );
    % z of qArm > 0, i.e., pointing outward from the cavity
    qArm = MMTK.FPparamQ( cavity.ITM.invROC, cavity.ETM.invROC, cavity.armIE.L );
    % change the direction of q, beam going from ITM to ETM
    qArm = MMTK.reverseDir( qArm );

    wITM = MMTK.q2W( qArm );
    wETM = MMTK.q2W( qArm + cavity.armIE.L );
end

% fill outputs
function fillOutputs( nargout_ )
    if nargout_ >= 4
        cavVal.rList = hkObj.rList;
        cavVal.etaArm = etaArm;

        cavVal.wITM = wITM;
        cavVal.wETM = wETM;

        cavVal.qArm = qArm;

        cavVal.lockedPhi = lockedPhi;
        cavVal.locked = locked;
    end
end

end
