%
% ModalModelTK : toolkit to calculate modal model related functions
%   needs HankelTK
%
% August 5, 2010 
%   Use invR (=1/R) instead of R as arguments.
%   f number is removed, except invRBack.
%
% June 24, 2010  Hiro Yamamoto  hiro@ligo.caltech.edu
%   first release
%

classdef MMTK < handle
    
    methods (Static)
        
        % effective curvature seen from back side of a mirror 
        % with refractive index n and internal f number invF
        % Rm > 0 for concave mirror
        % Use this for the second argument of FPparam to 
        % calculate parameters of a Michelson cavity
        function invRback = invROCback( invRm, n, invF )
            invRback = - n * invRm + invF;
        end

        % modal paramters in a cavity defined by curvatures, f number in mirror 2
        % and the cavity length.
        % returns Rayleigh range and the front mirror z
        function [z, z0] = FPparam( invROC1, invROC2, Lcav )
            g1 = 1 - Lcav * invROC1;
            g2 = 1 - Lcav * invROC2;
            z = Lcav * g2*(1 - g1)/(g1 + g2 - 2*g1*g2);
            z0 = Lcav * sqrt( g1*g2*(1 - g1*g2) ) / abs( g1 + g2 - 2*g1*g2 );
        end
        
        function q = FPparamQ( invROC1, invROC2, Lcav )
            [z, z0] = MMTK.FPparam( invROC1, invROC2, Lcav );
            q = z + 1i * z0;
        end
        
        % beam size on front mirror
        function w = w1FP( invROC1, invROC2, Lcav )
            [z, z0] = MMTK.FPparam( invROC1, invROC2, Lcav );
            w0 = sqrt(2*z0/CavityTK.kLam);
            w = w0*sqrt( 1 + (z/z0)^2 );
        end

        % gouy phase of the FP cavity
        function eta = eta00FP( invROC1, invROC2, Lcav )
            [z, z0] = MMTK.FPparam( invROC1, invROC2, Lcav );
            eta = atan2( Lcav - z, z0 ) + atan2( z, z0 );
        end
        
        % gouy phase by propagation distance
        function eta = gouyL( q, L )
            eta = atan2( real(q)+L, imag(q) ) - atan2( real(q), imag(q) );
        end

% ABCD matrix toolkit
        
        % ABCD operation on q
        function qp = ABCD_q( M, q )
            qp = ( M(1,1) * q + M(1,2) ) ./ ( M(2,1) * q + M(2,2) );
        end
        
        % ABCD operation on 1/q
        function qinvp = ABCD_qinv( M, qinv )
            qinvp = ( M(2,1) + M(2,2)*qinv ) ./ ( M(1,1) + M(1,2)*qinv );
        end
        
        % ABCD operation on q
        function [zp, z0p] = ABCD_zz0( M, z, z0 )
            qp = ABCDq( M, z + 1i*z0 );
            zp = real( qp );
            z0p = imag( qp );
        end
        
        % ABCD operation on (1/R and w)
        function [invRp, wp] = ABCD_rw( M, invR, w )
            qinvp = ABCDqinv( M, invR - 1i*CavityTK.kLam*2/w^2 );
            invRp = real( qinvp );
            wp = sqrt( -2*CavityTK.kLam / imag(qinvp) );
        end
        
        % propagation through a substrate with refractive index n
        function M = Prop_ABCD( L, n )
            if nargin == 1
                n = 1;
            end
            M = [ 1, L/n; 0, 1 ];
        end
        
        % lens specified by f number
        function M = LensF_ABCD( finv )
            M = [ 1, 0; -finv, 1 ];
        end
        
        % lens through a substrate with refractive index n and 
        % one side ROC invROC ( positive for concave )
        function M = LensR_ABCD( invR, n )
            if nargin == 1
                n = CavityTK.nSubs;
            end
            M = MMTK.LensF_ABCD( -(n-1)*invR );
        end
        
        % reflection with angle of incidence
        % plane = 0 : tangential (in the plane of incidence)
        %       = 1 : sagittal (perpendicular to plane of incidence)
        function M = Refl_ABCD( invR, theta, plane )
            if nargin < 3
                plane = 0;
            end
            if nargin < 2
                theta = 0;
            end
            if plane == 0
                cosCor = cos( theta );
            else
                cosCor = 1/cos( theta );
            end
            M = [ 1, 0; -2*invR/cosCor, 1];
        end
        
% conversion among various forms
 
        function [z, z0] = q2zz0( q )
            z = real(q);
            z0 = imag(q);
        end
        
        % q to 1/R and w
        function [invR, w] = q2RW( q )
            invR = real(1/q);
            w = MMTK.q2W(q);
        end
        
        % w from q
        function w = q2W( q )
            w = sqrt( -2/(CavityTK.kLam*imag(1/q)) );
        end
        
        % reverse direction ( q or invq will work )
        function qp = reverseDir( q )
            qp = -real( q ) + 1i*imag( q );
        end
    end
end
